<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2017 Wolf Utz <utz@riconet.de>, riconet
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace Riconet\RicoProducts\Controller;

use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use Riconet\RicoProducts\Utility\ImportUtility;
use Riconet\RicoProducts\Utility\ParseCSV;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
use TYPO3\CMS\Extbase\Domain\Model\Category;
use Riconet\RicoProducts\Domain\Model\Product;
use Riconet\RicoProducts\Domain\Model\Variant;
use Riconet\RicoProducts\Domain\Model\Attribute;
use Riconet\RicoProducts\Domain\Model\AttributeData;

/**
 * Class ImportController.
 */
class ImportController extends ActionController
{
    /**
     * productRepository.
     *
     * @var \Riconet\RicoProducts\Domain\Repository\ProductRepository
     * @inject
     */
    protected $productRepository = null;

    /**
     * variantRepository.
     *
     * @var \Riconet\RicoProducts\Domain\Repository\VariantRepository
     * @inject
     */
    protected $variantRepository = null;

    /**
     * categoryRepository.
     *
     * @var \TYPO3\CMS\Extbase\Domain\Repository\CategoryRepository
     * @inject
     */
    protected $categoryRepository = null;

    /**
     * attributeDataRepository.
     *
     * @var \Riconet\RicoProducts\Domain\Repository\AttributeDataRepository
     * @inject
     */
    protected $attributeDataRepository = null;

    /**
     * attributeRepository.
     *
     * @var \Riconet\RicoProducts\Domain\Repository\AttributeRepository
     * @inject
     */
    protected $attributeRepository = null;

    /**
     * fileReferenceRepository.
     *
     * @var \Riconet\RicoProducts\Domain\Repository\FileReferenceRepository
     * @inject
     */
    protected $fileReferenceRepository = null;

    /**
     * persistence.
     *
     * @var array
     */
    protected $persistence = null;

    /**
     * persistenceManager.
     *
     * @var array
     */
    protected $persistenceManager = null;

    /**
     * The index where attributes start.
     *
     * @var int
     */
    protected $attrubutesStartIndex = 12;

    /**
     * The import log.
     *
     * @var array
     */
    private $log = array(
        'skipped' => array(
            'categories' => array(),
            'subcategories' => array(),
            'attributes' => array(),
            'products' => array(),
            'variants' => array(),
            'accessories' => array(),
            'images' => array(),
        ),
        'inserted' => array(
            'categories' => array(),
            'subcategories' => array(),
            'attributes' => array(),
            'products' => array(),
            'variants' => array(),
            'accessories' => array(),
            'images' => array(),
        ),
        'updated' => array(
            'categories' => array(),
            'subcategories' => array(),
            'attributes' => array(),
            'products' => array(),
            'variants' => array(),
            'accessories' => array(),
            'images' => array(),
        ),
        'errors' => array(
            'categories' => array(),
            'subcategories' => array(),
            'attributes' => array(),
            'products' => array(),
            'variants' => array(),
            'accessories' => array(),
            'images' => array(),
        ),
    );

    /**
     * Action initialize.
     */
    public function initializeAction()
    {
        // Get the persistence.
        $this->persistence = $this->configurationManager->getConfiguration(
            ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK
        )['persistence'];
        // Get the persitence manager.
        $this->persistenceManager = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Persistence\\Generic\\PersistenceManager');
    }

    /**
     * action index.
     */
    public function indexAction()
    {
        $options = ImportUtility::generateImportSelectOptions();
        $this->view->assign('options', $options);
    }

    /**
     * action import.
     */
    public function importAction()
    {
        $args = $this->request->getArguments();
        $allowOverride = intval($args['importSettings']['allowOverwrite']);
        $showLog = intval($args['importSettings']['showLog']);
        $csv = ImportUtility::createCSVObject($args['importFile']);
        // Import categories.
        $this->importCategories($csv, $allowOverride);
        // Import Attributes.k
        $this->importAttributeDatas($csv);
        // Import Variants.
        $this->importVariants($csv, $allowOverride);
        // Import Products.
        $this->importProducts($csv, $allowOverride);
        $this->addFlashMessage('Imported!');
        if ($showLog) {
            $this->view->assign('log', $this->log);
        } else {
            $this->redirect('index');
        }
    }

    /**
     * Imports all categories.
     *
     * @param \Riconet\RicoProducts\Utility\ParseCSV $csv
     * @param bool                                   $allowOverride
     */
    public function importCategories(ParseCSV $csv, $allowOverride = false)
    {
        // Create top categories.
        foreach ($csv->data as $row) {
            // Check if the row has categories.
            if (!empty($row['categories'])) {
                $title = $row['categories'];
                if (!$this->categoryExist($title)) {
                    $category = new Category();
                    $category->setTitle($title);
                    $this->categoryRepository->add($category);
                    $this->persistenceManager->persistAll();
                    $this->log['inserted']['categories'][] = $title;
                } elseif ($allowOverride) {
                    $category = $this->categoryRepository->findByTitle($title)->getFirst();
                    $category->setTitle($title);
                    $this->categoryRepository->update($category);
                    $this->persistenceManager->persistAll();
                    $this->log['updated']['categories'][] = $title;
                } else {
                    // skiped...
                    $this->log['skipped']['categories'][] = $title;
                }
                // Create sub categories.
                if (!empty($row['subcategories'])) {
                    $titles = ImportUtility::splitToArray($row['subcategories']);
                    $parent = $this->categoryRepository->findByTitle($title)->getFirst();
                    foreach ($titles as $title) {
                        if (!$this->categoryExist($title)) {
                            $category = new Category();
                            $category->setTitle($title);
                            $category->setParentCategory($parent);
                            $this->categoryRepository->add($category);
                            $this->persistenceManager->persistAll();
                            $this->log['inserted']['subcategories'][] = $title;
                        } elseif ($allowOverride) {
                            // override, if allowed....
                            $category = $this->categoryRepository->findByTitle($title)->getFirst();
                            $category->setTitle($title);
                            $category->setParentCategory($parent);
                            $this->categoryRepository->update($category);
                            $this->persistenceManager->persistAll();
                            $this->log['updated']['subcategories'][] = $title;
                        } else {
                            // skiped...
                            $this->log['skipped']['subcategories'][] = $title;
                        }
                    }
                }
            }
        }
    }

    /**
     * Imports all attributeDatas.
     *
     * @param \Riconet\RicoProducts\Utility\ParseCSV $csv
     */
    public function importAttributeDatas(ParseCSV $csv)
    {
        // Create top categories.
        $index = 0;
        foreach ($csv->titles as $col) {
            // Check it is a attribute col.
            if ($index >= $this->attrubutesStartIndex) {
                if (!empty($col)) {
                    if (!$this->attributeDataExist($col)) {
                        $attributeData = new AttributeData();
                        $attributeData->setData($col);
                        $this->attributeDataRepository->add($attributeData);
                        $this->persistenceManager->persistAll();
                        $this->log['inserted']['attributes'][] = $col;
                    } else {
                        // skiped...
                         $this->log['skipped']['attributes'][] = $col;
                    }
                }
            }
            ++$index;
        }
    }

    /**
     * Imports all variants.
     *
     * @param \Riconet\RicoProducts\Utility\ParseCSV $csv
     * @param bool                                   $allowOverride
     */
    public function importVariants(ParseCSV $csv, $allowOverride = false)
    {
        // Create top categories.
        foreach ($csv->data as $row) {
            // Check if the row is a variant.
            if ($row['is_variant'] === '1') {
                $articleNumber = trim(str_replace(' ', '', $row['article_number']));
                $title = $row['title']; //addVariantRelatedAttributes
                if (!empty($articleNumber)) {
                    if (!$this->variantExist($articleNumber)) {
                        $variant = new Variant();
                        $variant = $this->addVariantRelatedAttributes($csv, $variant, $row);
                        $variant->setTitle($title);
                        $variant->setArticleNumber($articleNumber);
                        $variant->setSubtitle($row['subtitle']);
                        $variant->setDescription($row['description']);
                        $variant = $this->AddVaraiantRelatedImages($variant, $row);
                        $this->variantRepository->add($variant);
                        $this->persistenceManager->persistAll();
                        $this->log['inserted']['variants'][] = $articleNumber.'-'.$title;
                    } elseif ($allowOverride) {
                        $variant = $this->variantRepository->findByArticleNumber($articleNumber)->getFirst();
                        $variant = $this->addVariantRelatedAttributes($csv, $variant, $row);
                        $variant->setTitle($row['title']);
                        $variant->setSubtitle($row['subtitle']);
                        $variant->setDescription($row['description']);
                        $variant = $this->AddVaraiantRelatedImages($variant, $row);
                        $this->variantRepository->update($variant);
                        $this->persistenceManager->persistAll();
                        $this->log['updated']['variants'][] = $articleNumber.'-'.$title;
                    } else {
                        // skiped...
                        $this->log['skipped']['variants'][] = $articleNumber.'-'.$title;
                    }
                } else {
                    // Error no $articleNumber
                    $this->log['errors']['variants'][] = $title;
                }
            }
        }
    }

    /**
     * Imports all products.
     *
     * @param \Riconet\RicoProducts\Utility\ParseCSV $csv
     * @param bool                                   $allowOverride
     */
    public function importProducts(ParseCSV $csv, $allowOverride = false)
    {
        // Create top categories.
        foreach ($csv->data as $row) {
            // Check if the row is a product.
            if ($row['is_variant'] === '0') {
                $articleNumber = trim(str_replace(' ', '', $row['article_number']));
                $title = $row['title'];
                if (!empty($articleNumber)) {
                    if (!$this->productExist($articleNumber)) {
                        $product = new Product();
                        $product = $this->addProductRelatedAttributes($csv, $product, $row);
                        $product->setTitle($row['title']);
                        $product->setArticleNumber($articleNumber);
                        $product->setSubtitle($row['subtitle']);
                        $product->setDescription($row['description']);
                        $product = $this->AddRelatedVariants($product, $row);
                        $product = $this->addRelatedCategories($product, $row);
                        $product = $this->AddProductRelatedImages($product, $row);
                        $product = $this->addRelatedPlanImages($product, $row);
                        $this->productRepository->add($product);
                        $this->persistenceManager->persistAll();
                        $this->log['inserted']['products'][] = $articleNumber.'-'.$title;
                    } elseif ($allowOverride) {
                        $product = $this->productRepository->findByArticleNumber($articleNumber)->getFirst();
                        $product = $this->addProductRelatedAttributes($csv, $product, $row);
                        $product->setTitle($row['title']);
                        $product->setSubtitle($row['subtitle']);
                        $product->setDescription($row['description']);
                        $product = $this->AddRelatedVariants($product, $row);
                        $product = $this->addRelatedCategories($product, $row);
                        $product = $this->AddProductRelatedImages($product, $row);
                        $product = $this->addRelatedPlanImages($product, $row);
                        $this->productRepository->update($product);
                        $this->persistenceManager->persistAll();
                        $this->log['updated']['products'][] = $articleNumber.'-'.$title;
                    } else {
                        // skiped...
                        $this->log['skipped']['products'][] = $articleNumber.'-'.$title;
                    }
                } else {
                    // Error no $articleNumber
                    $this->log['errors']['products'][] = $title;
                }
            }
        }
        // Add Accessories.
        foreach ($csv->data as $row) {
            // Check if the row is a product.
            if ($row['is_variant'] === '0' && !empty($row['accessories'])) {
                $articleNumber = trim(str_replace(' ', '', $row['article_number']));
                if (!empty($articleNumber)) {
                    $product = $this->productRepository->findByArticleNumber($articleNumber)->getFirst();
                    $product = $this->addRelatedAccessories($product, $row);
                    $this->productRepository->update($product);
                    $this->persistenceManager->persistAll();
                } else {
                    // Error no $articleNumber
                    $this->log['errors']['accessories'][] = $articleNumber;
                }
            }
        }
    }

    /**
     * Checks if a categorie exist.
     *
     * @param string $title
     *
     * @return int
     */
    private function categoryExist($title)
    {
        return count($this->categoryRepository->findByTitle($title)->getFirst());
    }

    /**
     * Checks if a categorie exist.
     *
     * @param string $data
     *
     * @return int
     */
    private function attributeDataExist($data)
    {
        return count($this->attributeDataRepository->findByData($data)->getFirst());
    }

    /**
     * Checks if a variant exist.
     *
     * @param string $articleNumber
     *
     * @return int
     */
    private function variantExist($articleNumber)
    {
        return count($this->variantRepository->findByArticleNumber($articleNumber)->getFirst());
    }

    /**
     * Checks if a product exist.
     *
     * @param string $articleNumber
     *
     * @return int
     */
    private function productExist($articleNumber)
    {
        return count($this->productRepository->findByArticleNumber($articleNumber)->getFirst());
    }

    /**
     * Add related Variants to a product.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Product $product
     * @param array                                      $row
     *
     * @return \Riconet\RicoProducts\Domain\Model\Product
     */
    private function AddRelatedVariants(Product $product, array $row)
    {
        $variantArticleNumbers = array_filter(ImportUtility::splitToArray($row['variants']));
        $product->removeAllVariants();
        foreach ($variantArticleNumbers as $articleNumber) {
            if (!empty($articleNumber)) {
                $articleNumber = trim(str_replace(' ', '', $articleNumber));
                $variant = $this->variantRepository->findByArticleNumber($articleNumber)->getFirst();
                if (!is_null($variant)) {
                    $product->addVariant($variant);
                }
            }
        }

        return $product;
    }

    /**
     * Add related Categories to a product.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Product $product
     * @param array                                      $row
     *
     * @return \Riconet\RicoProducts\Domain\Model\Product
     */
    private function addRelatedCategories(Product $product, array $row)
    {
        $categoriesTitles = array_filter(array_merge(ImportUtility::splitToArray($row['categories']), ImportUtility::splitToArray($row['subcategories'])));
        $product->removeAllCategories();
        if (count($categoriesTitles) > 0) {
            foreach ($categoriesTitles as $title) {
                if (!empty($title)) {
                    $category = $this->categoryRepository->findByTitle($title)->getFirst();
                    if (!is_null($category)) {
                        $product->addCategory($category);
                    }
                }
            }
        }

        return $product;
    }

    /**
     * Add related Accessories to a product.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Product $product
     * @param array                                      $row
     *
     * @return \Riconet\RicoProducts\Domain\Model\Product
     */
    private function addRelatedAccessories(Product $product, array $row)
    {
        $accessoriesArticleNumbers = array_filter(ImportUtility::splitToArray($row['accessories']));
        $product->removeAllAccessories();
        if (count($accessoriesArticleNumbers) > 0) {
            foreach ($accessoriesArticleNumbers as $articleNumber) {
                if (!empty($articleNumber)) {
                    $articleNumber = trim(str_replace(' ', '', $articleNumber));
                    $accessory = $this->variantRepository->findByArticleNumber($articleNumber)->getFirst();
                    if (!is_null($accessory)) {
                        $product->addAccessory($accessory);
                    }
                }
            }
        }

        return $product;
    }

    /**
     * Add related Attrbutes and its Data to a product.
     *
     * @param \Riconet\RicoProducts\Utility\ParseCSV     $csv
     * @param \Riconet\RicoProducts\Domain\Model\Product $product
     * @param array                                      $row
     *
     * @return \Riconet\RicoProducts\Domain\Model\Product
     */
    private function addProductRelatedAttributes(ParseCSV $csv, Product $product, array $row)
    {
        $attributedatas = ImportUtility::getArrayIndexFromTill($this->attrubutesStartIndex, count($csv->titles), $csv->titles);
        $oldAttributes = $product->getAttributes();
        if (count($oldAttributes) > 0) {
            $product->removeAllAttributes();
            foreach ($oldAttributes as $attribute) {
                $this->attributeRepository->remove($attribute);
            }
        }
        foreach ($attributedatas as $attributedata) {
            if (!empty($row[$attributedata])) {
                $attributeData = $this->attributeDataRepository->findByData($attributedata)->getFirst();
                $attribute = new Attribute();
                $attribute->setTitle($attributeData->getUid());
                $attribute->setData($row[$attributedata]);
                // Set the right pid.
                $attribute->setPid($product->getPid());
                $this->attributeRepository->add($attribute);
                $product->addAttribute($attribute);
            }
        }

        return $product;
    }

    /**
     * Add related Attrbutes and its Data to a varaint.
     *
     * @param \Riconet\RicoProducts\Utility\ParseCSV     $csv
     * @param \Riconet\RicoProducts\Domain\Model\Variant $variant
     * @param array                                      $row
     *
     * @return \Riconet\RicoProducts\Domain\Model\Variant
     */
    private function addVariantRelatedAttributes(ParseCSV $csv, Variant $variant, array $row)
    {
        $attributedatas = ImportUtility::getArrayIndexFromTill($this->attrubutesStartIndex, count($csv->titles), $csv->titles);
        $oldAttributes = $variant->getAttributes();
        if (count($oldAttributes) > 0) {
            $variant->removeAllAttributes();
            foreach ($oldAttributes as $attribute) {
                $this->attributeRepository->remove($attribute);
            }
        }
        foreach ($attributedatas as $attributedata) {
            if (!empty($row[$attributedata])) {
                $attributeData = $this->attributeDataRepository->findByData($attributedata)->getFirst();
                $attribute = new Attribute();
                $attribute->setTitle($attributeData->getUid());
                $attribute->setData($row[$attributedata]);
                // Set the right pid.
                $attribute->setPid($variant->getPid());
                $this->attributeRepository->add($attribute);
                $this->persistenceManager->persistAll();
                $variant->addAttribute($attribute);
            }
        }

        return $variant;
    }

    /**
     * Adds a image file relation to a product.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Product $product
     * @param array                                      $row
     *
     * @return \Riconet\RicoProducts\Domain\Model\Product
     */
    private function AddProductRelatedImages(Product $product, array $row)
    {
        $imagesTitles = array_filter(ImportUtility::splitToArray($row['images']));
        $product->removeAllImages();
        if (count($imagesTitles) > 0) {
            $storageRepository = $this->objectManager->get('TYPO3\\CMS\\Core\\Resource\\StorageRepository');
            $storage = $storageRepository->findByUid('1');
            $targetFolder = ($storage->hasFolder('rico-products') ? $storage->getFolder('rico-products') : $storage->createFolder('rico-products'));
            foreach ($imagesTitles as $fileName) {
                if (!empty($fileName)) {
                    // File exist.
                    if ($targetFolder->hasFile($fileName)) {
                        $files = $targetFolder->getFiles();
                        $file = $files[$fileName];
                        $fileReference = $this->objectManager->get('Riconet\\RicoProducts\\Domain\\Model\\FileReference');
                        $fileReference->setFile($file);
                        $product->addImage($fileReference);
                    } else {
                        // Image does not exist!
                        $this->log['errors']['images'][] = 'File: '.$fileName.' NOT FOUND!';
                    }
                }
            }
        }

        return $product;
    }

    /**
     * Adds a image file relation to a variant.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Variant $variant
     * @param array                                      $row
     *
     * @return \Riconet\RicoProducts\Domain\Model\Variant
     */
    private function AddVaraiantRelatedImages(Variant $variant, array $row)
    {
        $imagesTitles = array_filter(ImportUtility::splitToArray($row['images']));
        $variant->removeAllImages();
        if (count($imagesTitles) > 0) {
            $storageRepository = $this->objectManager->get('TYPO3\\CMS\\Core\\Resource\\StorageRepository');
            $storage = $storageRepository->findByUid('1');
            $targetFolder = ($storage->hasFolder('rico-products') ? $storage->getFolder('rico-products') : $storage->createFolder('rico-products'));
            foreach ($imagesTitles as $fileName) {
                if (!empty($fileName)) {
                    // File exist.
                    if ($targetFolder->hasFile($fileName)) {
                        $files = $targetFolder->getFiles();
                        $file = $files[$fileName];
                        $fileReference = $this->objectManager->get('Riconet\\RicoProducts\\Domain\\Model\\FileReference');
                        $fileReference->setFile($file);
                        $variant->addImage($fileReference);
                    } else {
                        // Image does not exist!
                        $this->log['errors']['images'][] = 'File: '.$fileName.' NOT FOUND!';
                    }
                }
            }
        }

        return $variant;
    }

    /**
     * Adds a image file relation to a product.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Product $product
     * @param array                                      $row
     *
     * @return \Riconet\RicoProducts\Domain\Model\Product
     */
    private function addRelatedPlanImages(Product $product, array $row)
    {
        $plansTitles = array_filter(ImportUtility::splitToArray($row['plan']));
        $product->removeAllPlanimages();
        if (count($plansTitles) > 0) {
            $storageRepository = $this->objectManager->get('TYPO3\\CMS\\Core\\Resource\\StorageRepository');
            $storage = $storageRepository->findByUid('1');
            $targetFolder = ($storage->hasFolder('rico-products') ? $storage->getFolder('rico-products') : $storage->createFolder('rico-products'));
            foreach ($plansTitles as $fileName) {
                if (!empty($fileName)) {
                    // File exist.
                    if ($targetFolder->hasFile($fileName)) {
                        $files = $targetFolder->getFiles();
                        $file = $files[$fileName];
                        $fileReference = $this->objectManager->get('Riconet\\RicoProducts\\Domain\\Model\\FileReference');
                        $fileReference->setFile($file);
                        $product->addPlanimage($fileReference);
                    } else {
                        // Image does not exist!
                        $this->log['errors']['images'][] = 'File: '.$fileName.' NOT FOUND!';
                    }
                }
            }
        }

        return $product;
    }
}
