<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2017 Wolf Utz <utz@riconet.de>, riconet
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace Riconet\RicoProducts\Domain\Repository;

use TYPO3\CMS\Extbase\Domain\Model\Category;

/**
 * Class ProductRepository.
 */
class ProductRepository extends \TYPO3\CMS\Extbase\Persistence\Repository
{
    /**
     * categoryRepository.
     *
     * @var \TYPO3\CMS\Extbase\Domain\Repository\CategoryRepository
     * @inject
     */
    protected $categoryRepository = null;

    /**
     * Finds a AttributeData by pid and title.
     *
     * @param int    $pid
     * @param string $title
     *
     * @return array|\TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    public function findByPidAndTitle($pid, $title)
    {
        $query = $this->createQuery();
        $query->getQuerySettings()->setRespectStoragePage(false);
        $query->matching(
            $query->logicalAnd(
                $query->equals('pid', $pid),
                $query->equals('title', $title)
            )
        );

        return $query->execute();
    }

    /**
     * Finds products by category.
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\Category $parent
     * @param int                                      $recursions
     *
     * @return array|\TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    public function findByCategory(Category $parent, $recursions = 0)
    {
        $categories = $this->getCategoriesRecursive($parent, $recursions);
        $query = $this->createQuery();
        $query->matching(
            $query->in('categories.uid', $categories)
        );

        return $query->execute();
    }

    /**
     * Find products by categories.
     *
     * @param array $categories
     *
     * @return array|\TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    public function findByCategories(array $categories)
    {
        $query = $this->createQuery();
        $query->matching(
            $query->in('categories.uid', $categories)
        );

        return $query->execute();
    }

    /**
     * Gets categories recursive by a parent category.
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\Category $parent
     * @param int                                      $recursions
     *
     * @return array
     */
    protected function getCategoriesRecursive($parent, $recursions)
    {
        $categories = [$parent->getUid()];
        $nextUids = [$parent->getUid()];
        while ($recursions > 0) {
            $newNextUids = [];
            foreach ($nextUids as $nextUid) {
                $childs = $this->categoryRepository->findByParent($nextUid);
                /** @var $child Category */
                foreach ($childs as $child) {
                    $categories[] = $child->getUid();
                    $newNextUids[] = $child->getUid();
                }
            }
            $nextUids = $newNextUids;
            --$recursions;
        }

        return $categories;
    }

    /**
     * Find filtered.
     *
     * @param array $arguments
     *
     * @return array|\TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    public function findFiltered(array $arguments)
    {
        $query = $this->createQuery();
        $constraints = array();
        // search by string.
        if (isset($arguments['search']) && !empty($arguments['search'])) {
            $search = $arguments['search'];
            $constraints[] = $query->logicalOr(
                $query->like('articleNumber', '%'.$search.'%'),
                $query->like('title', '%'.$search.'%')
            );
        }
        // search by category_uid.
        if (isset($arguments['categories']) && !empty($arguments['categories'])) {
            $categoryUid = $arguments['categories'];
            $constraints[] = $query->contains('categories.uid', $categoryUid);
        }
        // Run...
        if (count($constraints) < 1) {
            // return $this->findAll();
        } else {
            $result = $query->matching(
                $query->logicalAnd(
                    $query->logicalAnd($constraints)
                )
            )->execute();

            return $result;
        }
    }
}
