<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2017 Wolf Utz <utz@riconet.de>, riconet
 *      Created on: 18.12.17 10:03
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace Riconet\RicoProducts\Hooks\KeSearch;

use TYPO3\CMS\Backend\Form\FormDataProvider\TcaSelectItems;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Database\Query\QueryBuilder;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Class DefaultHook.
 */
class DefaultHook
{
    /**
     * The key of the indexer.
     *
     * @var string
     */
    const KEY = 'rico_products_default';

    /**
     * The indexer title.
     *
     * @var string
     */
    const TITLE = 'Products indexer - Default';

    /**
     * The main table of the indexer.
     *
     * @var string
     */
    const TABLE = 'tx_ricoproducts_domain_model_product';

    /**
     * The prefix for displaing records in the front end.
     *
     * @var string
     */
    const FE_PREFIX = '[Products indexer]';

    /**
     * Adds the custom indexer to the TCA of indexer configurations, so that
     * it's selectable in the backend as an indexer type when you create a
     * new indexer configuration.
     *
     * @param array          $params
     * @param TcaSelectItems $pObj
     */
    public function registerIndexerConfiguration(&$params, TcaSelectItems $pObj)
    {
        $customIndexer = [
            self::TITLE,
            self::KEY,
            'EXT:rico_products/Resources/Public/Icons/RicoProducts.svg',
        ];
        $params['items'][] = $customIndexer;
    }

    /**
     * Custom indexer for ke_search.
     *
     * @param array                $indexerConfig Configuration from TYPO3 Backend
     * @param \tx_kesearch_indexer $indexerObject reference to indexer class
     *
     * @return string Message containing indexed elements
     */
    public function customIndexer(&$indexerConfig, \tx_kesearch_indexer &$indexerObject)
    {
        // Check indexer type.
        if ($indexerConfig['type'] != self::KEY) {
            return null;
        }
        $statement = $this->getStatement($indexerConfig['sysfolder']);
        $resultCount = $statement->rowCount();
        // Validate result count is greater than 0.
        if ($resultCount <= 0) {
            return null;
        }
        $insertCounter = 0;
        // Iterate over the result records.
        /** @var array $record */
        while ($record = $statement->fetch()) {
            // Add new index record.
            $indexerObject->storeInIndex(
                $indexerConfig['storagepid'],
                self::FE_PREFIX.strip_tags($record['title']),
                self::KEY,
                $indexerConfig['targetpid'],
                $this->getFullContent($record),
                '',
                $this->getParameters($record),
                strip_tags($record['teaser']),
                $record['sys_language_uid'],
                $record['starttime'],
                $record['endtime'],
                $record['fe_group'],
                false,
                $this->getAdditionalFields($record)
            );
            ++$insertCounter;
        }
        $indexerTitle = $indexerConfig['title'];

        return "<p><b>Custom Indexer \"$indexerTitle\": $insertCounter Elements have been indexed.</b></p>";
    }

    /**
     * Gets the statement.
     *
     * @param string $storagePageIds
     *
     * @return \Doctrine\DBAL\Driver\Statement|int
     */
    protected function getStatement(string $storagePageIds)
    {
        /** @var QueryBuilder $queryBuilder */
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable(self::TABLE);

        return $queryBuilder->select('*')
            ->from(self::TABLE)
            ->where(
                $queryBuilder->expr()->andX(
                    $queryBuilder->expr()->in('pid', $queryBuilder->createNamedParameter($storagePageIds)),
                    $queryBuilder->expr()->eq('hidden', $queryBuilder->createNamedParameter(false)),
                    $queryBuilder->expr()->eq('deleted', $queryBuilder->createNamedParameter(false))
                )
            )
            ->execute();
    }

    /**
     * Gets the additional fields array.
     *
     * @param array $record
     *
     * @return array
     */
    protected function getAdditionalFields(array $record): array
    {
        return [
            'orig_uid' => $record['uid'],
            'orig_pid' => $record['pid'],
            'sortdate' => $record['datetime'],
        ];
    }

    /**
     * Gets the parameter string.
     *
     * @param array $record
     *
     * @return string
     */
    protected function getParameters(array $record): string
    {
        // Add parameters for link generation.
        $uid = $record['uid'];
        $parameters = "&tx_ricoproducts_products[product]=$uid";
        $parameters .= '&tx_ricoproducts_products[controller]=Product';
        $parameters .= '&tx_ricoproducts_products[action]=show';

        return $parameters;
    }

    /**
     * Gets the full content to index.
     *
     * @param array $record
     *
     * @return string
     */
    protected function getFullContent(array $record): string
    {
        $contentCollection = [];
        $contentCollection[] = strip_tags($record['title']);
        $contentCollection[] = strip_tags($record['subtitle']);
        $contentCollection[] = strip_tags($record['teaser']);
        $contentCollection[] = strip_tags($record['article_number']);
        $contentCollection[] = strip_tags($record['description']);

        return $this->implodeWithGlue($contentCollection, "\n");
    }

    /**
     * Implodes an array with a given glue.
     *
     * @param array  $array
     * @param string $glue
     *
     * @return string
     */
    protected function implodeWithGlue(array $array, string $glue): string
    {
        $result = '';
        for ($i = 0; $i < count($array); ++$i) {
            if (is_string(strval($array[$i]))) {
                $result .= strval($array[$i]);
                if (($i - 1) < count(($array))) {
                    $result .= $glue;
                }
            }
        }

        return $result;
    }
}
