<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2017 Wolf Utz <utz@riconet.de>, riconet
 *      Created on: 18.12.17 10:03
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace Riconet\RicoProducts\Hooks\KeSearch;

use TYPO3\CMS\Backend\Form\FormDataProvider\TcaSelectItems;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Database\Query\QueryBuilder;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Service\FlexFormService;
use Doctrine\DBAL\Driver\Statement;

/**
 * Class NoShowViewHook.
 */
class NoShowViewHook
{
    /**
     * The key of the indexer.
     *
     * @var string
     */
    const KEY = 'rico_products_noshowview';

    /**
     * The indexer title.
     *
     * @var string
     */
    const TITLE = 'Products indexer - No show view';

    /**
     * The main table of the indexer.
     *
     * @var string
     */
    const TABLE = 'tx_ricoproducts_domain_model_product';

    /**
     * The prefix for displaing records in the front end.
     *
     * @var string
     */
    const FE_PREFIX = '[Products indexer]';

    protected $matrix = [];

    /**
     * Adds the custom indexer to the TCA of indexer configurations, so that
     * it's selectable in the backend as an indexer type when you create a
     * new indexer configuration.
     *
     * @param array          $params
     * @param TcaSelectItems $pObj
     */
    public function registerIndexerConfiguration(&$params, TcaSelectItems $pObj)
    {
        $customIndexer = [
            self::TITLE,
            self::KEY,
            'EXT:rico_products/Resources/Public/Icons/RicoProducts.svg',
        ];
        $params['items'][] = $customIndexer;
    }

    /**
     * Custom indexer for ke_search.
     *
     * @param array                $indexerConfig Configuration from TYPO3 Backend
     * @param \tx_kesearch_indexer $indexerObject reference to indexer class
     *
     * @return string Message containing indexed elements
     */
    public function customIndexer(&$indexerConfig, \tx_kesearch_indexer &$indexerObject)
    {
        // Check indexer type.
        if ($indexerConfig['type'] != self::KEY) {
            return null;
        }
        $statement = $this->getStatement($indexerConfig['sysfolder']);
        $resultCount = $statement->rowCount();
        // Validate result count is greater than 0.
        if ($resultCount <= 0) {
            return null;
        }
        $insertCounter = 0;
        $this->matrix = $this->getCategoryPageProductMatrix();
        // Iterate over the result records.
        /** @var array $record */
        while ($record = $statement->fetch()) {
            $categoryUid = $this->getFirstCategoryUid($record);
            if (!$categoryUid > 0) {
                continue;
            }
            $pid = $this->getPidOfMatrixByCategory($categoryUid);
            if (!$pid > 0) {
                continue;
            }
            // Add new index record.
            $indexerObject->storeInIndex(
                $indexerConfig['storagepid'],
                self::FE_PREFIX.strip_tags($record['title']),
                self::KEY,
                $pid.$this->getAnchor($record),
                $this->getFullContent($record),
                $this->getTags(),
                "",
                strip_tags($record['teaser']),
                $record['sys_language_uid'],
                $record['starttime'],
                $record['endtime'],
                $record['fe_group'],
                false,
                $this->getAdditionalFields($record)
            );
            ++$insertCounter;
        }
        $indexerTitle = $indexerConfig['title'];

        return "<p><b>Custom Indexer \"$indexerTitle\": $insertCounter Elements have been indexed.</b></p>";
    }

    /**
     * @param int $categoryUid
     *
     * @return int
     */
    protected function getPidOfMatrixByCategory(int $categoryUid)
    {
        foreach($this->matrix as $row) {
            if (intval($row["categoryUid"]) === $categoryUid) {
                return intval($row["pid"]);
            }
        }
        return 0;
    }


    /**
     * Gets the first category of the product.
     *
     * @param array $productRecord
     *
     * @return int
     */
    protected function getFirstCategoryUid(array $productRecord) : int
    {
        /** @var QueryBuilder $queryBuilder */
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('sys_category_record_mm');
        /** @var Statement $statement */
        $statement = $queryBuilder
            ->select('*')
            ->from('sys_category_record_mm')
            ->where(
                $queryBuilder->expr()->andX(
                    $queryBuilder->expr()->eq('uid_foreign', $queryBuilder->createNamedParameter($productRecord['uid'])),
                    $queryBuilder->expr()->eq('tablenames', $queryBuilder->createNamedParameter(self::TABLE))
                )
            )
            ->execute();
        if ($statement instanceof Statement && $statement->rowCount() > 0) {
            while ($record = $statement->fetch()) {
                return $record['uid_local'];
            }
        }

        return 0;
    }

    /**
     * Gets all products plugin flex form configurations.
     *
     * @return array
     */
    protected function getCategoryPageProductMatrix() : array
    {
        $configurations = [];
        /** @var QueryBuilder $queryBuilder */
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('tt_content');
        /** @var Statement $statement */
        $statement = $queryBuilder
            ->select('*')
            ->from('tt_content')
            ->where(
                $queryBuilder->expr()->andX(
                    $queryBuilder->expr()->eq('list_type', $queryBuilder->createNamedParameter("ricoproducts_products")),
                    $queryBuilder->expr()->eq('hidden', $queryBuilder->createNamedParameter(false)),
                    $queryBuilder->expr()->eq('deleted', $queryBuilder->createNamedParameter(false))
                )
            )
            ->execute();

        if ($statement instanceof Statement && $statement->rowCount() > 0) {
            while ($record = $statement->fetch()) {
                /** @var FlexFormService $FlexFormService */
                $FlexFormService = GeneralUtility::makeInstance(FlexFormService::class);
                $flexForm = $FlexFormService->convertFlexFormContentToArray($record['pi_flexform']);
                $categoryUid = intval($flexForm['settings']['categoryLimit']);
                $configurations[] = [
                    "categoryUid" => $categoryUid,
                    "pid" => $record['pid']
                ];
            }
        }

        return $configurations;
    }

    /**
     * Gets the statement.
     *
     * @param string $storagePageIds
     *
     * @return \Doctrine\DBAL\Driver\Statement|int
     */
    protected function getStatement(string $storagePageIds)
    {
        /** @var QueryBuilder $queryBuilder */
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable(self::TABLE);

        return $queryBuilder->select('*')
            ->from(self::TABLE)
            ->where(
                $queryBuilder->expr()->andX(
                    $queryBuilder->expr()->in('pid', $queryBuilder->createNamedParameter($storagePageIds)),
                    $queryBuilder->expr()->eq('hidden', $queryBuilder->createNamedParameter(false)),
                    $queryBuilder->expr()->eq('deleted', $queryBuilder->createNamedParameter(false))
                )
            )
            ->execute();
    }

    /**
     * Gets the additional fields array.
     *
     * @param array $record
     *
     * @return array
     */
    protected function getAdditionalFields(array $record) : array
    {
        return [
            'orig_uid' => $record['uid'],
            'orig_pid' => $record['pid'],
            'sortdate' => $record['datetime'],
        ];
    }

    /**
     * @param array $record
     *
     * @return string
     */
    public function getAnchor(array $record) : string
    {
        // Add parameters for link generation.
        $uid = $record['uid'];
        $parameters = "#product-$uid";

        return $parameters;
    }

    /**
     * Gets the full content to index.
     *
     * @param array $record
     *
     * @return string
     */
    protected function getFullContent(array $record) : string
    {
        $contentCollection = [];
        $contentCollection[] = strip_tags($record['title']);
        $contentCollection[] = strip_tags($record['subtitle']);
        $contentCollection[] = strip_tags($record['teaser']);
        $contentCollection[] = strip_tags($record['article_number']);
        $contentCollection[] = strip_tags($record['description']);

        return $this->implodeWithGlue($contentCollection, "\n");
    }

    /**
     * Implodes an array with a given glue.
     *
     * @param array  $array
     * @param string $glue
     *
     * @return string
     */
    protected function implodeWithGlue(array $array, string $glue) : string
    {
        $result = '';
        for ($i = 0; $i < count($array); ++$i) {
            if (is_string(strval($array[$i]))) {
                $result .= strval($array[$i]);
                if (($i - 1) < count(($array))) {
                    $result .= $glue;
                }
            }
        }

        return $result;
    }

    /**
     * Gets the tag string.
     *
     * @return string
     */
    protected function getTags() : string
    {
        return '#A#,#B#';
    }
}
